/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
/*-----------------------------------------------------------------------------
 *
 * $Author: brianpc.huang $
 * $Date: 2015/05/29 $
 * $RCSfile: ir_hw.c,v $
 * $Revision: #1 $
 *
 *---------------------------------------------------------------------------*/

/** @file ir_hw.c
 *  ir_hw.c provides IRRX/IRTX register access functions.
 */


//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include "x_hal_arm.h"

// local header files
#include "ir_reg.h"
#include "ir_io.h"
#include "ir_debug.h"
#include "ir_if.h"

// external header files
#include "x_lint.h"
#include "x_assert.h"
#include "x_bim.h"
#include "x_ckgen.h"
#include "x_pdwnc.h"
#include "x_printf.h"
#include "x_debug.h"

//-----------------------------------------------------------------------------
// Constant definitions
//-----------------------------------------------------------------------------

#define MAX_IRRX_DATA       (4)

#ifdef CC_MTK_LOADER
#define IRRX_NON_OSAI
#endif /* CC_MTK_LOADER */

//-----------------------------------------------------------------------------
// Static variables
//-----------------------------------------------------------------------------

#ifndef IRRX_NON_OSAI
static PFN_IRRXCB_T _pfnIrRxCbFunc = NULL;
#endif /* IRRX_NON_OSAI */


//-----------------------------------------------------------------------------
// Static functions
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
/** _IrRxHwBusyClr() to clear IRRX h/w buffer then it can receive next message.
 */
//-----------------------------------------------------------------------------
static void _IrRxHwBusyClr(void)
{
    vIO32WriteFldAlign(PDWNC_IRCLR, IRRX_CLR_BSY_IR, FLD_IRCLR);
}
#ifdef RCMM
#define RCMM_BUFFER_LEN 17
UINT32 u4IrPulseTimeUnit = 7; //use default value 7 us
UINT32 u4CheckData[RCMM_BUFFER_LEN];
#else
static UINT32 u4IrPulseTimeUnit = 50; //use default value 50 us
#endif
#ifndef IRRX_NON_OSAI
//-----------------------------------------------------------------------------
/** _IrRxHwIsr() the interrupt of IRRX from PDWNC interrupt ISR.
 *  @param u2VctId It must be PDWNC_INTNO_IRRX.
 */
//-----------------------------------------------------------------------------
static void _IrRxHwIsr(UINT16 u2VctId)
{
    UINT32 u4Info;
    UINT32 au4IrRxData[MAX_IRRX_DATA];
	UINT32 i, u4PulsePeriod;
    UNUSED(u2VctId);
	UNUSED(u4PulsePeriod);

    /* read IRRX IRQ data. */
    u4Info = u4IO32Read4B(PDWNC_IRCNT);
    au4IrRxData[0] = u4IO32Read4B(PDWNC_IRDATA0);
    au4IrRxData[1] = u4IO32Read4B(PDWNC_IRDATA1);


    LOG(7, "PWD:0x%08x RxIsr Info:0x%08x 0x%08x%08x\n",
            u4IO32Read4B(PDWNC_INTSTA),
            u4Info, au4IrRxData[1], au4IrRxData[0]);

	LOG(9, "IR period(us):\n");
	for(i = 0; i < 67; )
	{
		u4PulsePeriod = u4IO32Read4B(PDWNC_CHKDATA0 + i);
		#ifdef RCMM
		//error handle
		if((i/4) >= RCMM_BUFFER_LEN)
		{
			LOG(9, "[error]IR u4CheckData out of range\n");
			break;
		}
		u4CheckData[(i/4)] = u4PulsePeriod;
		#endif
		LOG(9, "%4d, %4d, %4d, %4d,",  (u4PulsePeriod & 0xff) * u4IrPulseTimeUnit, ((u4PulsePeriod >> 8 )& 0xff) * u4IrPulseTimeUnit,((u4PulsePeriod >> 16 )& 0xff) * u4IrPulseTimeUnit, ((u4PulsePeriod >> 24 )& 0xff) * u4IrPulseTimeUnit);
		i+=4;
		u4PulsePeriod = u4IO32Read4B(PDWNC_CHKDATA0 + i);
		#ifdef RCMM
		u4CheckData[(i/4)] = u4PulsePeriod;
		#endif
		LOG(9, " %4d, %4d, %4d, %4d\n",  (u4PulsePeriod & 0xff) * u4IrPulseTimeUnit, ((u4PulsePeriod >> 8 )& 0xff) * u4IrPulseTimeUnit,((u4PulsePeriod >> 16 )& 0xff) * u4IrPulseTimeUnit, ((u4PulsePeriod >> 24 )& 0xff) * u4IrPulseTimeUnit);
		i+=4;
	}

    if ((u4Info != 0) && (_pfnIrRxCbFunc != NULL))
    {
    	#ifdef RCMM
		_pfnIrRxCbFunc(u4Info, (UINT8 *)u4CheckData);
		#else
        _pfnIrRxCbFunc(u4Info, (UINT8 *)au4IrRxData);
		#endif
    }

    /* clear interrupt. */
    VERIFY(PDWNC_ClearInt(PDWNC_INTNO_IRRX));

    /* Do BIM clear IR busy bit, after interrupt clear */
    _IrRxHwBusyClr();
}

#if 0
#ifdef CC_CLI
//-----------------------------------------------------------------------------
/** _IrTxHwIsr() the interrupt of IRTX from BIM IRQ.
 *  @param u2VctId It must be VECTOR_IRTX.
 */
//-----------------------------------------------------------------------------
static void _IrTxHwIsr(UINT16 u2VctId)
{
    static INT32 i4Count = 0;

    UNUSED(u2VctId);
    /* clear interrupt. */
    VERIFY(BIM_ClearIrq(VECTOR_IRTX));
    LOG(0,"%s(%d) Cnt:%d\n", __FUNCTION__, __LINE__, i4Count++);
}
#endif /* CC_CLI */
#endif
#endif /* IRRX_NON_OSAI */

//-----------------------------------------------------------------------------
/** _IRHW_RxSetIsr() to enable/disable interrupt of IRRX
 *  IRRX interrupt is a sub-interrupt of PDWNC interrupt, the enable/disable \n
 *  function is to call PDWNC function to set the PDWNC interrupt enable \n
 *  register.
 *  @param fgSet a boolean value, true to enable ISR, false to disable.
 *  @retval 0 is operating successfully, otherwise failed.
 */
//-----------------------------------------------------------------------------
static INT32 _IRHW_RxSetIsr(BOOL fgSet)
{
#ifndef IRRX_NON_OSAI
    INT32 i4Val;
    void (* pfnOldIsr)(UINT16);

    if (fgSet)
    {   /* enable irq. */
        i4Val = PDWNC_RegIsr(PDWNC_INTNO_IRRX, _IrRxHwIsr, &pfnOldIsr);
        if (i4Val != PDWNC_OK)
        {
            return IR_FAIL;
        }

        VERIFY(PDWNC_ClearInt(PDWNC_INTNO_IRRX));
    }
    else
    {   /* disable irq. */
        i4Val = PDWNC_RegIsr(PDWNC_INTNO_IRRX, NULL, &pfnOldIsr);
        if (i4Val != OSR_OK)
        {
            return IR_FAIL;
        }
    }
#endif /* IRRX_NON_OSAI */
    return IR_SUCC;
}

#ifndef IRRX_NON_OSAI
#if 0
#ifdef CC_CLI
//-----------------------------------------------------------------------------
/** _IRHW_TxSetIsr() to enable/disable interrupt of IRTX
 *  @param fgSet a boolean value, true to enable ISR, false to disable.
 *  @retval 0 is operating successfully, otherwise failed.
 */
//-----------------------------------------------------------------------------
static INT32 _IRHW_TxSetIsr(BOOL fgSet)
{
    INT32 i4Val;
    void (* pfnOldIsr)(UINT16);

    if (fgSet)
    { /* enable irq. */
        i4Val = x_reg_isr(VECTOR_IRTX, _IrTxHwIsr, &pfnOldIsr);
        if (i4Val != OSR_OK)
        {
            return IR_FAIL;
        }
        VERIFY(BIM_ClearIrq(VECTOR_IRTX));
    }
    else
    { /* disable irq. */
        i4Val = x_reg_isr(VECTOR_IRTX, NULL, &pfnOldIsr);
        if (i4Val != OSR_OK)
        {
            return IR_FAIL;
        }
    }
    return IR_SUCC;
}
#endif /* CC_CLI */
#endif
#endif /* IRRX_NON_OSAI */


//-----------------------------------------------------------------------------
// Inter-file functions
//-----------------------------------------------------------------------------

/* HWIRRX export functions */

#if 0
//-----------------------------------------------------------------------------
/** IRHW_RxHwState() to get the IRRX internal state machine
 */
//-----------------------------------------------------------------------------
UINT32 IRHW_RxHwState()
{
    UINT32 u4Val, u4Val1, u4Val2;
    CRIT_STATE_T rState;
    /*
    rState = x_crit_start();
    u4Val1 = CKGEN_READ32(REG_PINMUX_SEL0);
    u4Val2 = BIM_READ32(REG_RW_DBG_MONSEL);
    CKGEN_WRITE32(REG_PINMUX_SEL0, 0x333c0);
    BIM_WRITE32(REG_RW_DBG_MONSEL, 0x60001);
    u4Val = CKGEN_READ32(REG_GPIO_IN1);
    u4Val = (u4Val >> 1) & 0x7;
    CKGEN_WRITE32(REG_PINMUX_SEL0, u4Val1);
    BIM_WRITE32(REG_RW_DBG_MONSEL, u4Val2);
    x_crit_end(rState);
    */
    return u4Val;
}
#endif

//-----------------------------------------------------------------------------
/** IRHW_RxRdConf() to read IRRX configration
 *  @param pi4Config to read PDWNC_IRCFGH
 *  @param pi4SaPeriod to read PDWNC_IRCFGL
 *  @param pi4Threshold to read PDWNC_IRTHD
 */
//-----------------------------------------------------------------------------
void IRHW_RxRdConf(INT32 *pi4Config, INT32 *pi4SaPeriod, INT32 *pi4Threshold)
{
    if ((pi4Config == NULL) || (pi4SaPeriod==NULL) || (pi4Threshold==NULL))
    {
        ASSERT(0);
        LINT_SUPPRESS_BRACE(527);
        return;
    }
    *pi4Config = (INT32)u4IO32Read4B(PDWNC_IRCFGH);
    *pi4SaPeriod = (INT32)u4IO32Read4B(PDWNC_IRCFGL);
    *pi4Threshold = (INT32)u4IO32Read4B(PDWNC_IRTHD);
}

//-----------------------------------------------------------------------------
/** IRHW_RxWrConf() to set IRRX configration
 *  @param pi4Config to read PDWNC_IRCFGH
 *  @param pi4SaPeriod to read PDWNC_IRCFGL
 *  @param pi4Threshold to read PDWNC_IRTHD
 */
//-----------------------------------------------------------------------------
void IRHW_RxWrConf(INT32 i4Config, INT32 i4SaPeriod, INT32 i4Threshold)
{
    /* Set IRRX clock to 3Mhz / 64 */
    vIO32WriteFldAlign(PDWNC_IRCKSEL, IRRX_CLK_3MHZ_DIV_64, FLD_IRCLKSEL);

    /* Config IRRX registers */
    vIO32Write4B(PDWNC_IRCFGH, (UINT32)i4Config);
    vIO32Write4B(PDWNC_IRCFGL, (UINT32)i4SaPeriod);
    vIO32Write4B(PDWNC_IRTHD, (UINT32)i4Threshold);

//for ir HW check pulse test
    //vIO32WriteFldAlign(PDWNC_IRCFGL, 0x546, FLD_CHK_SAPERIOD);
	if(IO32ReadFldAlign(PDWNC_PDMODE, FLD_DEEP_SLEEP))// 3Mhz
	{
		vIO32WriteFldAlign(PDWNC_IRCFGL, u4IrPulseTimeUnit * 3, FLD_CHK_SAPERIOD);
	}
	else
	{
		#if defined(CC_MT5881) || defined(CC_MT5399) || defined(CC_MT5882) || defined(CC_MT5883)
		if(IS_XTAL_24MHZ())
		{
			vIO32WriteFldAlign(PDWNC_IRCFGL, u4IrPulseTimeUnit * 24, FLD_CHK_SAPERIOD);
		}
		else if(IS_XTAL_27MHZ())
		#endif
		{
			vIO32WriteFldAlign(PDWNC_IRCFGL, u4IrPulseTimeUnit * 27, FLD_CHK_SAPERIOD);
		}
	}

    vIO32WriteFldAlign(PDWNC_IRCFGH, 1, FLD_CHK_EN);
}

//-----------------------------------------------------------------------------
/** IRHW_Init() to set IRRX configration and start IRRX ISR
 *  @param pi4Config to read PDWNC_IRCFGH
 *  @param pi4SaPeriod to read PDWNC_IRCFGL
 *  @param pi4Threshold to read PDWNC_IRTHD
 *  @retval 0 is operating successfully, otherwise failed.
 */
//-----------------------------------------------------------------------------
INT32 IRHW_RxInit(INT32 i4Config, INT32 i4SaPeriod, INT32 i4Threshold)
{
#ifndef CC_UBOOT
    UINT32 u4State;
#endif /* CC_UBOOT */
    INT32 i, i4Ret;

    IRHW_RxWrConf(i4Config, i4SaPeriod, i4Threshold);
    i4Ret = _IRHW_RxSetIsr((BOOL)TRUE);

    LOG(0,"u4IrPulseTimeUnit is %u us\n", u4IrPulseTimeUnit);

#ifndef CC_UBOOT
    u4State = HalCriticalStart();
#endif /* CC_UBOOT */

    // wait 1ms for Busy Clr operation finish.
    for (i=0; i<30000; i++) { }

    _IrRxHwBusyClr();

    // wait 1ms for Busy Clr operation finish.
    for (i=0; i<30000; i++) { }

#ifndef CC_UBOOT
    HalCriticalEnd(u4State);
#endif /* CC_UBOOT */

//    LOG(0,"After IRRX State: %d\n", IRHW_RxHwState());

    return i4Ret;
}

//-----------------------------------------------------------------------------
/** IRHW_Stop() to stop IRRX ISR
 *  @retval 0 is operating successfully, otherwise failed.
 */
//-----------------------------------------------------------------------------
INT32 IRHW_RxStop()
{
    INT32 i4Ret;
    i4Ret = _IRHW_RxSetIsr((BOOL)FALSE);
    return i4Ret;
}

//-----------------------------------------------------------------------------
/** IRHW_Start() to enable IRRX ISR
 *  @retval 0 is operating successfully, otherwise failed.
 */
//-----------------------------------------------------------------------------
INT32 IRHW_RxStart()
{
    INT32 i4Ret;
    i4Ret = _IRHW_RxSetIsr((BOOL)TRUE);
    return i4Ret;
}

#ifndef IRRX_NON_OSAI
//-----------------------------------------------------------------------------
/** IRHW_RxSetCallback() to register interrupt callback routine.
 *  @param pfnCallback the new register callback routine
 *  @param ppfnOld pointer to the old callback routine
 *  @retval 0 is operating successfully, otherwise failed.
 */
//-----------------------------------------------------------------------------
INT32 IRHW_RxSetCallback(PFN_IRRXCB_T pfnCallback, PFN_IRRXCB_T *ppfnOld)
{
    if (ppfnOld != NULL)
    {
        *ppfnOld = _pfnIrRxCbFunc;
    }
    _pfnIrRxCbFunc = pfnCallback;
    return IR_SUCC;
}


#ifdef CC_CLI
/* HWIRTX export functions */

//-----------------------------------------------------------------------------
/** IRHW_TxRdConf() to read IRTX configuration
 *  @param pi4Config
 *  @param prWave
 *  @param pi4Modulation
 */
//-----------------------------------------------------------------------------
void IRHW_TxRdConf(INT32 *pi4Config, IRTXWAVE_T *prWave, INT32 *pi4Modulation)
{
    if ((pi4Config == NULL) || (prWave==NULL) || (pi4Modulation==NULL))
    {
        return;
    }
    *pi4Config = (INT32) (IRTX_READ32(IRTX_CONFIG_REG) & IRTX_INIT_CONF_MASK);
    prWave->u4SyncHigh = IRTX_READ32(IRTX_SYNC_HIGH_REG);
    prWave->u4SyncLow = IRTX_READ32(IRTX_SYNC_LOW_REG);
    prWave->u4Data0High = IRTX_READ32(IRTX_DATA0_HIGH_REG);
    prWave->u4Data0Low = IRTX_READ32(IRTX_DATA0_LOW_REG);
    prWave->u4Data1High = IRTX_READ32(IRTX_DATA1_HIGH_REG);
    prWave->u4Data1Low = IRTX_READ32(IRTX_DATA1_LOW_REG);
    *pi4Modulation = (INT32) IRTX_READ32(IRTX_MODULATION_REG);
}

//-----------------------------------------------------------------------------
/** IRHW_TxRdConf() to read IRTX configuration
 *  @param i4Config
 *  @param prWave
 *  @param i4Modulation
 */
//-----------------------------------------------------------------------------
void IRHW_TxWrConf(INT32 i4Config, const IRTXWAVE_T *prWave, INT32 i4Modulation)
{
    UINT32 u4Val;

    u4Val = IRTX_READ32(IRTX_CONFIG_REG);
    u4Val &= IRTX_CONF_BITNUM_MASK;
    IRTX_WRITE32(IRTX_CONFIG_REG,
                (((UINT32)i4Config & IRTX_INIT_CONF_MASK) | u4Val));

    ASSERT(prWave!=NULL);
    IRTX_WRITE32(IRTX_SYNC_HIGH_REG, prWave->u4SyncHigh);
    IRTX_WRITE32(IRTX_SYNC_LOW_REG, prWave->u4SyncLow);
    IRTX_WRITE32(IRTX_DATA0_HIGH_REG, prWave->u4Data0High);
    IRTX_WRITE32(IRTX_DATA0_LOW_REG, prWave->u4Data0Low);
    IRTX_WRITE32(IRTX_DATA1_HIGH_REG, prWave->u4Data1High);
    IRTX_WRITE32(IRTX_DATA1_LOW_REG, prWave->u4Data1Low);
//    IRTX_WRITE32(IRTX_SYNC_HIGH_REG, 0x3b4c0);
//    IRTX_WRITE32(IRTX_SYNC_LOW_REG, 0x1da60);
//    IRTX_WRITE32(IRTX_DATA0_HIGH_REG, 0x3b10);
//    IRTX_WRITE32(IRTX_DATA0_LOW_REG, 0x3b10);
//    IRTX_WRITE32(IRTX_DATA1_HIGH_REG, 0x3b10);
//    IRTX_WRITE32(IRTX_DATA1_LOW_REG, 0xb23e);

    IRTX_WRITE32(IRTX_MODULATION_REG, (UINT32)i4Modulation);
//    VERIFY(_IRHW_TxSetIsr(TRUE)==IR_SUCC);
}

//-----------------------------------------------------------------------------
/** IRHW_TxSendData() to read IRTX configuration
 *  @param *pi4DataArray the transferred data array
 *  @param i4BitNum the bitnumber of data array to transferred
 */
//-----------------------------------------------------------------------------
void IRHW_TxSendData(const INT32 *pi4DataArray, INT32 i4BitNum)
{
    UINT32 i, u4Val;

    if ((pi4DataArray == NULL) || (i4BitNum == 0))
    {
        return;
    }

    /* must check trigger bit is 0, and then we can send the data. */
    do
    {
        u4Val = IRTX_READ32(IRTX_CONFIG_REG);
    } while (u4Val & IRTX_CONF_START_SEND);

    /* Modify bit number first, in RC5/RC6, with bitnum 0,
     * it will send data automatically. */
    u4Val = IRTX_READ32(IRTX_CONFIG_REG);
    u4Val &= ~IRTX_CONF_BITNUM_MASK;
    u4Val |= ((UINT32)(i4BitNum & 0x7f) << 8);
    IRTX_WRITE32(IRTX_CONFIG_REG, u4Val);

    /* ((i4BitNum+31) >> 5) means how many i4ords to send. */
    for (i=0; i < ((UINT32)(i4BitNum+31) >> 5); i++)
    {
        IRTX_WRITE32((IRTX_DATA_BYTE_3_0+(i*4)), ((UINT32)pi4DataArray[i]));
    }

    /* Then trigger the data send, don't do bit number
     * modification and triggerring at the same time. */
    u4Val |= IRTX_CONF_START_SEND;
    IRTX_WRITE32(IRTX_CONFIG_REG, u4Val);
}
#endif /* CC_CLI */
#endif /* IRRX_NON_OSAI */

