/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
/*-----------------------------------------------------------------------------
 *
 * $Author$
 * $Date$
 * $RCSfile: ir_io.h,v $
 * $Revision$
 *
 *---------------------------------------------------------------------------*/


/** @file ir_io.h
 *  ir_io.h The IRRX/IRTX register access macros
 */

#ifndef IR_IO_H
#define IR_IO_H

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include "x_typedef.h"
#include "x_hal_5381.h"


//-----------------------------------------------------------------------------
// Configurations
//-----------------------------------------------------------------------------

#ifdef CC_MINI_DRIVER
#define IRHW_DEBUG  0
#else
#define IRHW_DEBUG  0
#endif /* IRHW_DEBUG */


//-----------------------------------------------------------------------------
// Macro definitions
//-----------------------------------------------------------------------------

// IR device register read/write macros
#if IRHW_DEBUG

#define IRRX_WRITE32(offset, value) IRRXDB_IoWr(offset, value, __FILE__, __LINE__)
#define IRRX_READ32(offset)         IRRXDB_IoRd(offset, __FILE__, __LINE__)
#define IRTX_WRITE32(offset, value) IRTXDB_IoWr(offset, value, __FILE__, __LINE__)
#define IRTX_READ32(offset)         IRTXDB_IoRd(offset, __FILE__, __LINE__)

#else /* #if IRHW_DEBUG */

//#define IRRX_WRITE32(offset, value) IO_WRITE32(PDWNC_BASE, (offset), (value))
//#define IRRX_READ32(offset)         IO_READ32(PDWNC_BASE, (offset))
#define IRTX_WRITE32(offset, value) IO_WRITE32(BIM_BASE, (offset), (value))
#define IRTX_READ32(offset)         IO_READ32(BIM_BASE, (offset))

#endif /* IRHW_DEBUG */


//-----------------------------------------------------------------------------
// Inter-file functions
//-----------------------------------------------------------------------------

EXTERN void IRRXDB_IoWr(UINT32 u4Offset, UINT32 u4Value, INT8 *szFile, INT32 Line);
EXTERN UINT32 IRRXDB_IoRd(UINT32 u4Offset, INT8 *szFile, INT32 Line);
EXTERN void IRTXDB_IoWr(UINT32 u4Offset, UINT32 u4Value, INT8 *szFile, INT32 Line);
EXTERN UINT32 IRTXDB_IoRd(UINT32 u4Offset, INT8 *szFile, INT32 Line);


#endif /* IR_IO_H */

